@@ -1,9 +1,9 @@
 package Net::DNS::RR::NS;
 
 #
-# $Id: NS.pm 1528 2017-01-18 21:44:58Z willem $
+# $Id: NS.pm 1597 2017-09-22 08:04:02Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1597 $)[1];
 
 
 use strict;
@@ -32,7 +32,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my $nsdname = $self->{nsdname} || return '';
+	my $nsdname = $self->{nsdname};
 	$nsdname->encode(@_);
 }
 
@@ -40,7 +40,7 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $nsdname = $self->{nsdname} || return '';
+	my $nsdname = $self->{nsdname};
 	$nsdname->string;
 }
 
