@@ -1,9 +1,9 @@
 package Net::DNS::RR::NSEC3;
 
 #
-# $Id: NSEC3.pm 1694 2018-07-16 04:19:40Z willem $
+# $Id: NSEC3.pm 1749 2019-07-21 09:15:55Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1694 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1749 $)[1];
 
 
 use strict;
@@ -48,7 +48,7 @@ my %digest = (
 		my $name = shift;
 		my $key	 = uc $name;				# synthetic key
 		$key =~ s /[\W_]//g;				# strip non-alphanumerics
-		$digestbyname{$key} || croak "unknown digest type $name";
+		$digestbyname{$key} || croak qq[unknown digest type "$name"];
 	}
 
 	sub _digestbyval {
@@ -64,13 +64,14 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 
 	my $limit = $offset + $self->{rdlength};
 	my $ssize = unpack "\@$offset x4 C", $$data;
-	@{$self}{qw(algorithm flags iterations saltbin)} = unpack "\@$offset CCnx a$ssize", $$data;
+	my ( $algorithm, $flags, $iterations, $saltbin ) = unpack "\@$offset CCnx a$ssize", $$data;
+	@{$self}{qw(algorithm flags iterations saltbin)} = ( $algorithm, $flags, $iterations, $saltbin );
 	$offset += 5 + $ssize;
 	my $hsize = unpack "\@$offset C", $$data;
 	$self->{hnxtname} = unpack "\@$offset x a$hsize", $$data;
 	$offset += 1 + $hsize;
 	$self->{typebm} = substr $$data, $offset, ( $limit - $offset );
-	$self->{hashfn} = _hashfn( @{$self}{qw(algorithm iterations saltbin)} );
+	$self->{hashfn} = _hashfn( $algorithm, $iterations, $saltbin );
 }
 
 
@@ -99,14 +100,14 @@ sub _format_rdata {			## format rdata portion of RR string.
 sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	$self->algorithm(shift);
+	my $alg = $self->algorithm(shift);
 	$self->flags(shift);
-	$self->iterations(shift);
+	my $iter = $self->iterations(shift);
 	my $salt = shift;
 	$self->salt($salt) unless $salt eq '-';
 	$self->hnxtname(shift);
 	$self->typelist(@_);
-	$self->{hashfn} = _hashfn( @{$self}{qw(algorithm iterations saltbin)} );
+	$self->{hashfn} = _hashfn( $alg, $iter, $self->{saltbin} );
 }
 
 
@@ -140,11 +141,9 @@ sub flags {
 
 
 sub optout {
-	my $bit = 0x01;
 	for ( shift->{flags} ) {
-		my $set = $bit | ( $_ ||= 0 );
-		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
-		return $_ & $bit;
+		$_ = ( shift() ? 0 : 0x01 ) ^ ( 0x01 | ( $_ || 0 ) ) if scalar @_;
+		return 0x01 & ( $_ || 0 );
 	}
 }
 
@@ -160,7 +159,7 @@ sub iterations {
 sub salt {
 	my $self = shift;
 	return unpack "H*", $self->saltbin() unless scalar @_;
-	$self->saltbin( pack "H*", map /[^\dA-F]/i ? croak "corrupt hex" : $_, join "", @_ );
+	$self->saltbin( pack "H*", join "", map { /^"*([\dA-Fa-f]*)"*$/ || croak("corrupt hex"); $1 } @_ );
 }
 
 
@@ -179,14 +178,24 @@ sub hnxtname {
 }
 
 
+sub match {
+	my ( $self, $name ) = @_;
+
+	my ($owner) = $self->{owner}->label;
+	my $ownerhash = _decode_base32hex($owner);
+
+	my $hashfn = $self->{hashfn};
+	$ownerhash eq &$hashfn($name);
+}
+
 sub covers {
 	my ( $self, $name ) = @_;
 
-	my ( $owner, @zone ) = $self->{owner}->_wire;
+	my ( $owner, @zone ) = $self->{owner}->label;
 	my $ownerhash = _decode_base32hex($owner);
 	my $nexthash  = $self->{hnxtname};
 
-	my @label = new Net::DNS::DomainName($name)->_wire;
+	my @label = new Net::DNS::DomainName($name)->label;
 	my @close = @label;
 	foreach (@zone) { pop(@close) }				# strip zone labels
 	return if lc($name) ne lc( join '.', @close, @zone );	# out of zone
@@ -204,29 +213,14 @@ sub covers {
 }
 
 
-sub covered {				## historical
-	&covers;						# uncoverable pod
-}
-
-sub match {				## historical
-	my ( $self, $name ) = @_;				# uncoverable pod
-
-	my ($owner) = $self->{owner}->_wire;
-	my $ownerhash = _decode_base32hex($owner);
-
-	my $hashfn = $self->{hashfn};
-	$ownerhash eq &$hashfn($name);
-}
-
-
 sub encloser {
 	my ( $self, $qname ) = @_;
 
-	my ( $owner, @zone ) = $self->{owner}->_wire;
+	my ( $owner, @zone ) = $self->{owner}->label;
 	my $ownerhash = _decode_base32hex($owner);
 	my $nexthash  = $self->{hnxtname};
 
-	my @label = new Net::DNS::DomainName($qname)->_wire;
+	my @label = new Net::DNS::DomainName($qname)->label;
 	my @close = @label;
 	foreach (@zone) { pop(@close) }				# strip zone labels
 	return if lc($qname) ne lc( join '.', @close, @zone );	# out of zone
@@ -234,14 +228,13 @@ sub encloser {
 	my $hashfn = $self->{hashfn};
 
 	my $encloser = $qname;
-	shift @label;
 	foreach (@close) {
 		my $nextcloser = $encloser;
-		my $hash = &$hashfn( $encloser = join '.', @label );
 		shift @label;
+		my $hash = &$hashfn( $encloser = join '.', @label );
 		next if $hash ne $ownerhash;
 		$self->{nextcloser} = $nextcloser;		# next closer name
-		$self->{wildcard} = join '.', '*', $encloser;	# wildcard at provable encloser
+		$self->{wildcard}   = "*.$encloser";		# wildcard at provable encloser
 		return $encloser;				# provable encloser
 	}
 	return;
@@ -287,8 +280,10 @@ sub _hashfn {
 		$class->new(@argument);
 	};
 	my $exception = $@;
+	return sub { croak $exception }
+			if $exception;
 
-	return $exception ? sub { croak $exception } : sub {
+	return sub {
 		my $name  = new Net::DNS::DomainName(shift)->canonical;
 		my $key	  = join '', $name, $key_adjunct;
 		my $cache = $$cache1{$key} ||= $$cache2{$key};	# two layer cache
@@ -434,6 +429,13 @@ interpolated into a string.
 typemap() returns a Boolean true value if the specified RRtype occurs
 in the type bitmap of the NSEC3 record.
 
+=head2 match
+
+    $matched = $rr->match( 'example.foo' );
+
+match() returns a Boolean true value if the hash of the domain name
+argument matches the hashed owner name of the NSEC3 RR.
+
 =head2 covers
 
     $covered = $rr->covers( 'example.foo' );
