@@ -1,9 +1,9 @@
 package Net::DNS::RR::OPT;
 
 #
-# $Id: OPT.pm 1578 2017-06-20 12:49:23Z willem $
+# $Id: OPT.pm 1754 2019-08-19 14:12:28Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1578 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1754 $)[1];
 
 
 use strict;
@@ -72,9 +72,8 @@ sub string {				## overide RR method
 	my $flags  = sprintf '%04x', $self->flags;
 	my $rcode  = $self->rcode;
 	my $size   = $self->size;
-	my @option = sort { $a <=> $b } $self->options;
-	my @lines  = map $self->_format_option($_), @option;
-	my @format = join "\n;;\t\t", @lines;
+	my @option = map join( "\n;;\t\t\t\t", $self->_format_option($_) ), $self->options;
+	my @format = join "\n;;\t\t", @option;
 
 	$rcode = 0 if $rcode < 16;				# weird: 1 .. 15 not EDNS codes!!
 
@@ -92,16 +91,15 @@ QQ
 }
 
 
-my ( $class, $ttl );
-
 sub class {				## overide RR method
-	carp qq[Usage: OPT has no "class" attribute, please use "size()"] unless $class++;
-	&size;
+	my $self = shift;
+	$self->_deprecate(qq[please use "size()"]);
+	$self->size(@_);
 }
 
 sub ttl {				## overide RR method
 	my $self = shift;
-	carp qq[Usage: OPT has no "ttl" attribute, please use "flags()" or "rcode()"] unless $ttl++;
+	$self->_deprecate(qq[please use "flags()" or "rcode()"]);
 	my @rcode = map unpack( 'C',   pack 'N', $_ ), @_;
 	my @flags = map unpack( 'x2n', pack 'N', $_ ), @_;
 	pack 'C2n', $self->rcode(@rcode), $self->version, $self->flags(@flags);
@@ -109,8 +107,10 @@ sub ttl {				## overide RR method
 
 
 sub version {
-	my $version = shift->{version};
-	return defined($version) ? $version : 0;
+	my $self = shift;
+
+	$self->{version} = 0 + shift if scalar @_;
+	$self->{version} || 0;
 }
 
 
@@ -144,7 +144,7 @@ sub flags {
 sub options {
 	my ($self) = @_;
 	my $options = $self->{option} || {};
-	return keys %$options;
+	my @options = sort { $a <=> $b } keys %$options;
 }
 
 sub option {
@@ -164,8 +164,9 @@ sub _format_option {
 	my $package = join '::', __PACKAGE__, $option;
 	$package =~ s/-/_/g;
 	my $defined = length($payload) && $package->can('_image');
-	my @payload = $defined ? eval { $package->_image($payload) } : unpack 'H*', $payload;
-	Net::DNS::RR::_wrap( "$option\t=> (", @payload, ')' );
+	my @element = $defined ? eval { $package->_image($payload) } : unpack 'H*', $payload;
+	my $protect = pop(@element);
+	Net::DNS::RR::_wrap( "$option\t=> (", map( "$_,", @element ), "$protect )" );
 }
 
 
@@ -188,21 +189,25 @@ sub _set_option {
 
 	my $options = $self->{option} ||= {};
 	delete $options->{$number};
-	if ( ref($value) || scalar(@etc) ) {
-		my $option = ednsoptionbyval($number);
+	return unless defined $value;
+	if ( ref($value) || scalar(@etc) || $value !~ /\D/ ) {
 		my @arg = ( $value, @etc );
 		@arg = @$value if ref($value) eq 'ARRAY';
 		@arg = %$value if ref($value) eq 'HASH';
 		if ( $arg[0] eq 'OPTION-DATA' ) {
 			$value = $arg[1];
 		} else {
+			my $option  = ednsoptionbyval($number);
 			my $package = join '::', __PACKAGE__, $option;
 			$package =~ s/-/_/g;
-			croak "unable to compose option $option" unless $package->can('_compose');
-			$value = $package->_compose(@arg);
+			if ( $package->can('_compose') ) {
+				$value = $package->_compose(@arg);
+			} elsif ( scalar(@etc) ) {
+				croak "unable to compose option $option";
+			}
 		}
 	}
-	$options->{$number} = $value if defined $value;
+	$options->{$number} = $value;
 }
 
 
@@ -230,10 +235,10 @@ sub _image { &_decompose; }
 
 
 package Net::DNS::RR::OPT::DHU;					# RFC6975
-use base qw(Net::DNS::RR::OPT::DAU);
+our @ISA = qw(Net::DNS::RR::OPT::DAU);
 
 package Net::DNS::RR::OPT::N3U;					# RFC6975
-use base qw(Net::DNS::RR::OPT::DAU);
+our @ISA = qw(Net::DNS::RR::OPT::DAU);
 
 
 package Net::DNS::RR::OPT::CLIENT_SUBNET;			# RFC7871
@@ -241,24 +246,27 @@ use Net::DNS::RR::A;
 use Net::DNS::RR::AAAA;
 
 my %family = qw(1 Net::DNS::RR::A	2 Net::DNS::RR::AAAA);
-my @field  = qw(FAMILY SOURCE-PREFIX-LENGTH SCOPE-PREFIX-LENGTH ADDRESS);
+my @field8 = qw(FAMILY SOURCE-PREFIX-LENGTH SCOPE-PREFIX-LENGTH ADDRESS);
 
 sub _compose {
 	my ( $class, %argument ) = @_;
 	my $address = bless( {}, $family{$argument{FAMILY}} )->address( $argument{ADDRESS} );
-	my $preamble = pack 'nC2', map $_ ||= 0, @argument{@field};
-	my $bitmask = $argument{'SOURCE-PREFIX-LENGTH'};
+	my $preamble = pack 'nC2', map $_ || 0, @argument{@field8};
+	my $bitmask  = $argument{'SOURCE-PREFIX-LENGTH'};
 	pack "a* B$bitmask", $preamble, unpack 'B*', $address;
 }
 
 sub _decompose {
 	my %hash;
-	@hash{@field} = unpack 'nC2a*', $_[1];
+	@hash{@field8} = unpack 'nC2a*', $_[1];
 	$hash{ADDRESS} = bless( {address => $hash{ADDRESS}}, $family{$hash{FAMILY}} )->address;
-	my @payload = map { ( $_ => $hash{$_} ) } @field;
+	my @payload = map { ( $_ => $hash{$_} ) } @field8;
 }
 
-sub _image { &_decompose; }
+sub _image {
+	my %hash  = &_decompose;
+	my @image = map "$_ => $hash{$_}", @field8;
+}
 
 
 package Net::DNS::RR::OPT::EXPIRE;				# RFC7314
@@ -272,23 +280,27 @@ sub _decompose {
 	my @payload = ( 'EXPIRE-TIMER' => unpack 'N', $_[1] );
 }
 
-sub _image { &_decompose; }
+sub _image { join ' => ', &_decompose; }
 
 
 package Net::DNS::RR::OPT::COOKIE;				# RFC7873
 
-my @key = qw(CLIENT-COOKIE SERVER-COOKIE);
+my @field10 = qw(CLIENT-COOKIE SERVER-COOKIE);
 
 sub _compose {
 	my ( $class, %argument ) = @_;
-	pack 'a8 a*', map $_ || '', @argument{@key};
+	pack 'a8 a*', map $_ || '', @argument{@field10};
 }
 
 sub _decompose {
 	my %hash;
-	my $template = ( length( $_[1] ) < 16 ) ? 'a8' : 'a8 a*';
-	@hash{@key} = unpack $template, $_[1];
-	my @payload = map { ( $_ => $hash{$_} ) } @key;
+	@hash{@field10} = unpack 'a8 a*', $_[1];
+	my @payload = map { ( $_ => $hash{$_} ) } @field10;
+}
+
+sub _image {
+	my %hash  = &_decompose;
+	my @image = map join( ' => ', $_, unpack 'H*', $hash{$_} ), @field10;
 }
 
 
@@ -300,10 +312,10 @@ sub _compose {
 }
 
 sub _decompose {
-	my @payload = ( TIMEOUT => unpack 'n', $_[1] );
+	my @payload = ( 'TIMEOUT' => unpack 'n', $_[1] );
 }
 
-sub _image { &_decompose; }
+sub _image { join ' => ', &_decompose; }
 
 
 package Net::DNS::RR::OPT::PADDING;				# RFC7830
@@ -318,7 +330,7 @@ sub _decompose {
 	my @payload = ( 'OPTION-LENGTH' => length( $_[1] ) );
 }
 
-sub _image { &_decompose; }
+sub _image { join ' => ', &_decompose; }
 
 
 package Net::DNS::RR::OPT::CHAIN;				# RFC7901
@@ -327,16 +339,16 @@ use Net::DNS::DomainName;
 sub _compose {
 	my ( $class, %argument ) = @_;
 	my ($trust_point) = values %argument;
-	Net::DNS::DomainName->new( $trust_point || return '' )->encode;
+	Net::DNS::DomainName->new($trust_point)->encode;
 }
 
 sub _decompose {
 	my ( $class, $payload ) = @_;
-	my $fqdn = Net::DNS::DomainName->decode( \$payload )->string;
+	my $fqdn    = Net::DNS::DomainName->decode( \$payload )->string;
 	my @payload = ( 'CLOSEST-TRUST-POINT' => $fqdn );
 }
 
-sub _image { &_decompose; }
+sub _image { join ' => ', &_decompose; }
 
 
 package Net::DNS::RR::OPT::KEY_TAG;				# RFC8145
@@ -374,7 +386,11 @@ __END__
     ;;	    flags:  8000
     ;;	    rcode:  NOERROR
     ;;	    size:   1280
-    ;;	    option: COOKIE  => ( 7261776279746573 )
+    ;;	    option: DAU	   => ( 8, 10, 13, 14, 15, 16 )
+    ;;		    DHU	   => ( 1, 2, 4 )
+    ;;		    COOKIE => ( CLIENT-COOKIE => 7261776279746573,
+    ;;				SERVER-COOKIE =>  )
+
 
 =head1 DESCRIPTION
 
@@ -399,9 +415,10 @@ other unpredictable behaviour.
 
 =head2 version
 
-	$version = $rr->version;
+    $version = $rr->version;
+    $rr->version( $version );
 
-The version of EDNS used by this OPT record.
+The version of EDNS supported by this OPT record.
 
 =head2 size
 
@@ -458,10 +475,10 @@ For the example above:
 
 	%hash = $packet->edns->option(10);
 
-	{
-	    'CLIENT-COOKIE' => 'rawbytes',
-	    'SERVER-COOKIE' => undef
-	};
+	%hash = (
+		'CLIENT-COOKIE' => 'rawbytes',
+		'SERVER-COOKIE' => ''
+		);
 
 
 For some options, an array is more appropriate:
@@ -469,20 +486,19 @@ For some options, an array is more appropriate:
 	@algorithms = $packet->edns->option(6);
 
 
-Similar forms of array syntax may be used to construct the option value:
+Similar forms of array or hash syntax may be used to construct the
+option value:
 
 	$packet->edns->option( DHU => [1, 2, 4] );
-	$packet->edns->option( 6   => (1, 2, 4) );
 
 	$packet->edns->option( COOKIE => {'CLIENT-COOKIE' => $cookie} );
-	$packet->edns->option( 10     => ('CLIENT-COOKIE' => $cookie) );
 
 
 =head1 COPYRIGHT
 
 Copyright (c)2001,2002 RIPE NCC.  Author Olaf M. Kolkman.
 
-Portions Copyright (c)2012,2017 Dick Franks.
+Portions Copyright (c)2012,2017-2019 Dick Franks.
 
 All rights reserved.
 
