@@ -1,9 +1,9 @@
 package Net::DNS::RR::SMIMEA;
 
 #
-# $Id: SMIMEA.pm 1597 2017-09-22 08:04:02Z willem $
+# $Id: SMIMEA.pm 1741 2019-04-16 13:10:38Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1597 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1741 $)[1];
 
 
 use strict;
@@ -89,7 +89,7 @@ sub matchingtype {
 sub cert {
 	my $self = shift;
 	return unpack "H*", $self->certbin() unless scalar @_;
-	$self->certbin( pack "H*", map /[^\dA-F]/i ? croak "corrupt hex" : $_, join "", @_ );
+	$self->certbin( pack "H*", join "", map { /^"*([\dA-Fa-f]*)"*$/ || croak("corrupt hex"); $1 } @_ );
 }
 
 
